/*
 * Implements DevReg::Device.
 */

#include "libdevreg++.h"
#include <cstdlib>
#include <cstdio>

using DevReg::DeviceNode;
using DevReg::Device;
using DevReg::ReadException;

DevReg::Device::Device(const string &d)
	throw (ReadException) :
	dir(d)
{
	const char *locations[] = {"internal", "external", "virtual", "unknown", 0};
	int i = 0;

	bus = readLine(d + "bus");
	deviceid = readLine(d + "deviceid");
	location_desc = readLine(d + "locationdesc");
	vendor = readLine(d + "vendor");
	model = readLine(d + "model");
	type = readLine(d + "type");
	location_type = (enum DevReg::LocationType) readEnum(d + "location", locations);

	parent_id = readLine(d + "parent");
	superdevice_id = readLine(d + "superdevice");

	while (exists(d + "devicenodes/" + i)) {
		string s = d + "devicenodes/" + i + "/";
		nodes.push_back(new DeviceNode(readLine(s + "interface"), 
					       readLine(s + "name")));
		i++;
	}
}

DevReg::Device::~Device() 
{
	vector<DeviceNode*>::iterator i = nodes.begin();
	while (i != nodes.end())
		delete *(i++);
}

const Device *DevReg::Device::getParent() const
{
	return parent;
}

const Device *DevReg::Device::getSuperDevice() const
{
	return superdevice;
}

const Device *DevReg::Device::getTopmostDevice() const
{
	const Device *d = this;
	while (d->superdevice)
		d = d->superdevice;
	return d;
}

const vector<Device*> &DevReg::Device::getAllSubDevices() const
{
	return subdevices;
}

const vector<Device*> &DevReg::Device::getAllChildren() const
{
	return children;
}

const vector<DeviceNode*> &DevReg::Device::getAllDeviceNodes() const
{
	return nodes;
}

const string &DevReg::Device::getDir() const
{
	return dir;
}

const string &DevReg::Device::getBus() const
{
	return bus;
}

const string &DevReg::Device::getType() const
{
	return type;
}

const string &DevReg::Device::getDeviceId() const
{
	return deviceid;
}

const string &DevReg::Device::getModel() const
{
	return model;
}

const string &DevReg::Device::getVendor() const
{
	return vendor;
}

const string &DevReg::Device::getLocation() const
{
	return location_desc;
}

DevReg::LocationType DevReg::Device::getLocationType() const
{
	return location_type;
}


