/*
 * Implements the C API for libdevreg. 
 */

#include "libdevreg.h"
#include "libdevreg++.h"
#include <cstdio>

using namespace DevReg;
using std::string;


devreg_t devreg_create_registry(const char *filename)
{
	try {
		if (filename)
			return (devreg_t) new DeviceRegistry(filename);
		else
			return (devreg_t) new DeviceRegistry();
	}
	catch(...) {
		return NULL;
	}
}

void devreg_free_registry(devreg_t self)
{
	delete (DeviceRegistry*) self;
}

int devreg_refresh(devreg_t self)
{
	try {
		((DeviceRegistry*) self)->refresh();
		return 1;
	}
	catch (...) {
		return 0;
	}
}

devreg_device_t devreg_get_device_by_id(devreg_t self,
					const char *deviceid, 
					int onlyExact,
					int multiMatch)
{
	return (devreg_device_t) 
		((DeviceRegistry*)self)->getDeviceById(deviceid,
						       onlyExact ? true:false,
						       multiMatch ? true:false);
}

int devreg_get_devices_by_id(devreg_t self,
			     const char *deviceid, 
			     devreg_device_t *devlist,
			     unsigned int devlist_size,
			     int onlyExact)
{
	vector<Device*> dl;
	((DeviceRegistry*)self)->getDevicesById(deviceid, 
						dl,
						onlyExact?true:false);
	if ((devlist_size > 0) && (dl.size()>0) && devlist) 
		for (unsigned int i = 0; 
		     i < ((devlist_size<dl.size())?devlist_size:dl.size());
		     i++) 
			devlist[i] = dl[i];

	return dl.size();
}

int devreg_get_devices_by_interface(devreg_t self,
				    const char *interface, 
				    devreg_device_t *devlist,
				    unsigned int devlist_size)
{
	vector<Device*> dl;
	((DeviceRegistry*)self)->getDevicesByInterface(interface, dl);
	if ((devlist_size > 0) && (dl.size()>0) && devlist) 
		for (unsigned int i = 0; 
		     i < ((devlist_size<dl.size())?devlist_size:dl.size());
		     i++) 
			devlist[i] = dl[i];

	return dl.size();
}

int devreg_get_devices_by_bus(devreg_t self,
			      const char *bus, 
			      devreg_device_t *devlist,
			      unsigned int devlist_size)
{
	vector<Device*> dl;
	if (bus)
		((DeviceRegistry*)self)->getDevicesByBus(bus, dl);
	else
		((DeviceRegistry*)self)->getAllRootDevices(dl);		
	if ((devlist_size > 0) && (dl.size()>0) && devlist) 
		for (unsigned int i = 0; 
		     i < ((devlist_size<dl.size())?devlist_size:dl.size());
		     i++) 
			devlist[i] = dl[i];

	return dl.size();
}

devreg_device_t devreg_device_get_parent(devreg_device_t self)
{
	return (devreg_device_t) ((Device*)self)->getParent();
}
	
devreg_device_t devreg_device_get_superdevice(devreg_device_t self)
{
	return (devreg_device_t) ((Device*)self)->getSuperDevice();
}
	
devreg_device_t devreg_device_get_topmost_device(devreg_device_t self)
{
	return (devreg_device_t) ((Device*)self)->getTopmostDevice();
}

const char *devreg_device_get_dir(devreg_device_t self)
{
	return ((Device*)self)->getDir().c_str();	
}

const char *devreg_device_get_bus(devreg_device_t self)
{
	return ((Device*)self)->getBus().c_str();	
}

const char *devreg_device_get_type(devreg_device_t self)
{
	return ((Device*)self)->getType().c_str();	
}

const char *devreg_device_get_deviceid(devreg_device_t self)
{
	return ((Device*)self)->getDeviceId().c_str();	
}

const char *devreg_device_get_model(devreg_device_t self)
{
	return ((Device*)self)->getModel().c_str();	
}

const char *devreg_device_get_vendor(devreg_device_t self)
{
	return ((Device*)self)->getVendor().c_str();	
}

const char *devreg_device_get_location(devreg_device_t self)
{
	return ((Device*)self)->getLocation().c_str();	
}

devreg_location_type_t devreg_device_get_location_type(devreg_device_t self)
{
	return (devreg_location_type_t) (int)((Device*)self)->getLocationType();	
}
	
int devreg_device_get_subdevice_num(devreg_device_t self)
{
	return ((Device*)self)->getAllSubDevices().size();
}
	
devreg_device_t devreg_device_get_subdevice(devreg_device_t self, 
					    unsigned int n)
{
	return (devreg_device_t) (((Device*)self)->getAllSubDevices()[n]);
}

int devreg_device_get_child_num(devreg_device_t self)
{
	return ((Device*)self)->getAllChildren().size();
}
	
devreg_device_t devreg_device_get_child(devreg_device_t self, 
					unsigned int n)
{
	return (devreg_device_t) (((Device*)self)->getAllChildren()[n]);
}
	
int devreg_device_get_devicenode_num(devreg_device_t self)
{
	return ((Device*)self)->getAllDeviceNodes().size();
}
	
devreg_devicenode_t devreg_device_get_devicenode(devreg_device_t self, 
						 unsigned int n)
{
	return (devreg_devicenode_t) (((Device*)self)->getAllDeviceNodes()[n]);
}

const char* devreg_devicenode_get_interface(devreg_devicenode_t self)
{
	return ((DeviceNode*)self)->getInterface().c_str();
}

const char *devreg_devicenode_get_filename(devreg_devicenode_t self)
{
	return ((DeviceNode*)self)->getFilename().c_str();
}

int devreg_read_integer(const char *filename, int *i) 
{
	try {
		*i = readInteger(filename);
	} 
	catch (...) {
		return 0;
	}
	return 1;
}

int devreg_read_ulong(const char *filename, unsigned long *u)
{
	try {
		*u = readULong(filename);
	} 
	catch (...) {
		return 0;
	}
	return 1;
}

int devreg_read_line(const char *filename, char *buf, int buflen)
{
	try {
		string s = readLine(filename);
		strncpy(buf, s.c_str(), buflen);
		if (s.length() >= buflen)
			buf[buflen-1] = 0;
	} 
	catch (...) {
		return 0;
	}
	return 1;
}

int devreg_read_enum(const char *filename, const char **values, int *e)
{
	try {
		*e = readEnum(filename, values);
	} 
	catch (...) {
		return 0;
	}
	return 1;
}

int devreg_file_exists(const char *filename)
{
	return exists(filename) ? 1 : 0;
}

int devreg_read_changecount(const char *filename, 
			    int *pfd,
			    int *changecount)
{
	try {
		*changecount = readChangecount(filename, pfd);
	} 
	catch (...) {
		return 0;
	}
	return 1;	
}
